@@ -59,7 +59,11 @@ class NameSpace:
 	pass
 
 class FileInfo:
-	__slots__ = ('fn', 'object_id_before_left', 'object_id_before_right', 'object_id_after')
+
+	# original_fn: the file name as printed by git, with special characters encoded as three digit octal numbers
+	# fn: the real file name
+
+	__slots__ = ('fn', 'original_fn', 'object_id_before_left', 'object_id_before_right', 'object_id_after')
 
 	def __init__(self, fn=None):
 		self.fn = fn
@@ -96,6 +100,8 @@ class Model(Runner):
 
 	command_functions = dict()
 
+	ignore_returncode = False
+
 
 	# ------- generate output -------
 
@@ -122,7 +128,7 @@ class Model(Runner):
 			args = cmd[1:]
 			return fun(self, args)
 
-		return self.run_and_get_output(cmd).replace('\r', self.CARRIAGE_RETURN).splitlines()
+		return self.run_and_get_output(cmd, ignore_returncode=self.ignore_returncode).replace('\r', self.CARRIAGE_RETURN).splitlines()
 
 
 	# ------- manipulate commands -------
@@ -654,8 +660,10 @@ class DetailsModel(Model):
 	min_line_number_width = 3
 
 	re_ansi_esscape_sequence = "(\[[0-9;]*m)?"
-	reo_start_new_file = re.compile(re_ansi_esscape_sequence + "diff (--git a/(?P<fn1>.*?) b/|--cc (?P<fn2>[^]*)|--combined (?P<fn3>[^]*))")
+	reo_start_new_file = re.compile(re_ansi_esscape_sequence + "diff (--git \"?a/(?P<fn1>.*?)\"? \"?b/|--cc \"?(?P<fn2>[^\"]*)\"?|--combined \"?(?P<fn3>[^\"]*))")
+	reo_escape = re.compile(r'(\\[0-7][0-7][0-7])+')
 	reo_object_id = re.compile(re_ansi_esscape_sequence + "index ((?P<beforeleft>[0-9a-f]+),)?(?P<before>[0-9a-f]+)\.\.(?P<after>[0-9a-f]+)")
+	reo_filename = re.compile(re_ansi_esscape_sequence + "(\+\+\+|---) ")
 	reo_linenumber = re.compile(re_ansi_esscape_sequence +
 		r"(@@@ -(?P<start_before_left>\d+)(,(?P<number_before_left>\d+))? -(?P<start_before_right>\d+)(,(?P<number_before_right>\d+))?"
 		r"|@@ -(?P<start_before>\d+)(,(?P<number_before>\d+))?) "
@@ -786,6 +794,14 @@ class DetailsModel(Model):
 		if self.cmd_untracked == ['$untracked']:
 			if self.untracked_relative == settings.RELATIVE_CWD:
 				path = os.path.abspath
+
+		for i in range(len(filenames)):
+			fn = filenames[i]
+			if fn.startswith('"') and fn.endswith('"'):
+				fn = fn[1:-1]
+			fn = self.decode_filename(fn)
+			filenames[i] = fn
+
 		return [([(settings.COLOR_DETAILS_UNTRACKED, utils.colored_str_to_markup(formatter.format(pattern, filename=fn), self.app.define_color))], TYPE_UNTRACKED, FileInfo(path(fn))) for fn in filenames]
 
 	def format_command_error(self, e, action=None, setting=None):
@@ -849,11 +865,16 @@ class DetailsModel(Model):
 				self.file_info.object_id_before_left = m.group('beforeleft')
 				self.file_info.object_id_before_right = m.group('before')
 				self.file_info.object_id_after = m.group('after')
+			elif self.reo_filename.match(ln):
+				if self.file_info.fn != self.file_info.original_fn:
+					ns.out[ns.i] = ns.out[ns.i].replace(self.file_info.original_fn, self.file_info.fn)
 
 
 		if self.insert_align_character:
-			ns.out[ns.i] = ALIGN_CHAR + ln
+			ns.out[ns.i] = ALIGN_CHAR + ns.out[ns.i]
 		if self.is_start_new_file(ln):
+			if self.file_info.fn != self.file_info.original_fn:
+				ns.out[ns.i] = ns.out[ns.i].replace(self.file_info.original_fn, self.file_info.fn)
 			ns.out[ns.i] = (ns.out[ns.i], TYPE_START_OF_FILE, self.file_info)
 
 	def _set_linenumber(self, ns, ns_name, match, re_name):
@@ -958,12 +979,35 @@ class DetailsModel(Model):
 				fn = m.group('fn2')
 				if fn is None:
 					fn = m.group('fn3')
+
+			original_fn = fn
+			fn = self.decode_filename(fn)
+
 			self.file_info = FileInfo()
 			self.file_info.fn = fn
+			self.file_info.original_fn = original_fn
+
 			return True
 		else:
 			return False
 
+	def decode_filename(self, fn: str) -> str:
+		def replace(m: re.Match) -> str:
+			original = m.group()
+			ls = original.split('\\')
+			assert ls[0] == ''
+			ls = ls[1:]
+			li = [int(n, base=8) for n in ls]
+			return bytes(li).decode()
+
+		if self.reo_escape.search(fn):
+			try:
+				fn = self.reo_escape.sub(replace, fn)
+			except unicodedecodeerror:
+				pass
+
+		return fn
+
 
 	# ------- command functions ------
 
@@ -1170,6 +1214,9 @@ class DiffModel(DetailsModel):
 
 	_diff_commands = ['cmd']
 
+	# with --exit-code (implied by --no-index) return code 1 does not mean error but file changed
+	ignore_returncode = True
+
 	def __init__(self, args):
 		self.hash_id = None
 		self.id_type = None
